@@ -4,11 +4,7 @@ import numpy as np
 from skimage.metrics import structural_similarity as ssim
 from imagehash import average_hash, phash, dhash
 from PIL import Image
-import time
-import pandas as pd
-import matplotlib.pyplot as plt
-import math
-from typing import Tuple
+from typing import Tuple, Optional
 
 logging.basicConfig(
     level=logging.INFO,
@@ -19,13 +15,13 @@ logger = logging.getLogger(__name__)
 class FrameComparator:
     """Base class for frame comparison methods."""
     
-    def compare(self, static_frame: np.ndarray, new_frame: np.ndarray, stream_key: str) -> Tuple[bool, float]:
+    def compare(self, static_frame: np.ndarray, new_frame: np.ndarray, stream_key: Optional[str] = None) -> Tuple[bool, float]:
         """Compare frames and determine if they are similar.
         
         Args:
             static_frame: Reference frame (RGB, cv2 image as np.ndarray).
             new_frame: New frame to compare (RGB, cv2 image as np.ndarray).
-            stream_key: Identifier for the video stream (e.g., camera ID).
+            stream_key: Optional identifier for the video stream (e.g., camera ID).
             
         Returns:
             Tuple[bool, float]: (is_similar, similarity_score)
@@ -48,7 +44,6 @@ class AbsDiffComparator(FrameComparator):
             if not isinstance(threshold, (int, float)) or threshold < 0:
                 msg = f"Invalid threshold: {threshold}, must be non-negative"
                 logger.error(msg)
-                 #print(msg)
                 raise ValueError(msg)
             self.threshold = threshold
         except Exception as exc:
@@ -56,32 +51,26 @@ class AbsDiffComparator(FrameComparator):
             print(f"Error initializing AbsDiffComparator: {str(exc)}")
             raise
 
-    def compare(self, static_frame: np.ndarray, new_frame: np.ndarray, stream_key: str) -> Tuple[bool, float]:
+    def compare(self, static_frame: np.ndarray, new_frame: np.ndarray, stream_key: Optional[str] = None) -> Tuple[bool, float]:
         """Compare frames using mean absolute difference.
         
         Args:
             static_frame: Reference frame (RGB, cv2 image as np.ndarray).
             new_frame: New frame to compare (RGB, cv2 image as np.ndarray).
-            stream_key: Identifier for the video stream (e.g., camera ID).
+            stream_key: Optional identifier for the video stream (e.g., camera ID).
             
         Returns:
             Tuple[bool, float]: (is_similar, mean_difference)
         """
         try:
+            stream_key = stream_key or "default_stream_key"
             if not isinstance(static_frame, np.ndarray) or not isinstance(new_frame, np.ndarray):
                 msg = f"Invalid frame type for stream {stream_key}: static_frame and new_frame must be np.ndarray"
                 logger.error(msg)
-                 #print(msg)
                 return False, 0.0
             if static_frame.shape != new_frame.shape or static_frame.shape[2] != 3:
                 msg = f"Frames for stream {stream_key} have mismatched dimensions or are not RGB"
                 logger.error(msg)
-                 #print(msg)
-                return False, 0.0
-            if not isinstance(stream_key, str) or not stream_key:
-                msg = f"Invalid stream_key for stream {stream_key}: must be a non-empty string"
-                logger.error(msg)
-                 #print(msg)
                 return False, 0.0
             
             diff = cv2.absdiff(static_frame, new_frame)
@@ -89,7 +78,6 @@ class AbsDiffComparator(FrameComparator):
             if np.isnan(mean_diff):
                 msg = f"NaN detected in AbsDiff comparison for stream {stream_key}"
                 logger.error(msg)
-                 #print(msg)
                 return False, 0.0
             return mean_diff < self.threshold, float(mean_diff)
         except cv2.error as exc:
@@ -121,7 +109,6 @@ class SSIMComparator(FrameComparator):
             if not isinstance(threshold, (int, float)) or not 0 <= threshold <= 1:
                 msg = f"Invalid threshold: {threshold}, must be between 0 and 1"
                 logger.error(msg)
-                 #print(msg)
                 raise ValueError(msg)
             self.threshold = threshold
         except Exception as exc:
@@ -129,32 +116,26 @@ class SSIMComparator(FrameComparator):
             print(f"Error initializing SSIMComparator: {str(exc)}")
             raise
 
-    def compare(self, static_frame: np.ndarray, new_frame: np.ndarray, stream_key: str) -> Tuple[bool, float]:
+    def compare(self, static_frame: np.ndarray, new_frame: np.ndarray, stream_key: Optional[str] = None) -> Tuple[bool, float]:
         """Compare frames using SSIM.
         
         Args:
             static_frame: Reference frame (RGB, cv2 image as np.ndarray).
             new_frame: New frame to compare (RGB, cv2 image as np.ndarray).
-            stream_key: Identifier for the video stream (e.g., camera ID).
+            stream_key: Optional identifier for the video stream (e.g., camera ID).
             
         Returns:
             Tuple[bool, float]: (is_similar, ssim_score)
         """
         try:
+            stream_key = stream_key or "default_stream_key"
             if not isinstance(static_frame, np.ndarray) or not isinstance(new_frame, np.ndarray):
                 msg = f"Invalid frame type for stream {stream_key}: static_frame and new_frame must be np.ndarray"
                 logger.error(msg)
-                 #print(msg)
                 return False, 0.0
             if static_frame.shape != new_frame.shape or static_frame.shape[2] != 3:
                 msg = f"Frames for stream {stream_key} have mismatched dimensions or are not RGB"
                 logger.error(msg)
-                 #print(msg)
-                return False, 0.0
-            if not isinstance(stream_key, str) or not stream_key:
-                msg = f"Invalid stream_key for stream {stream_key}: must be a non-empty string"
-                logger.error(msg)
-                 #print(msg)
                 return False, 0.0
             
             gray_static = cv2.cvtColor(static_frame, cv2.COLOR_BGR2GRAY)
@@ -163,7 +144,6 @@ class SSIMComparator(FrameComparator):
             if np.isnan(score):
                 msg = f"NaN detected in SSIM comparison for stream {stream_key}"
                 logger.error(msg)
-                 #print(msg)
                 return False, 0.0
             return score > self.threshold, float(score)
         except cv2.error as exc:
@@ -195,7 +175,6 @@ class AverageHashComparator(FrameComparator):
             if not isinstance(threshold, int) or threshold < 0:
                 msg = f"Invalid threshold: {threshold}, must be a non-negative integer"
                 logger.error(msg)
-                 #print(msg)
                 raise ValueError(msg)
             self.threshold = threshold
         except Exception as exc:
@@ -203,32 +182,26 @@ class AverageHashComparator(FrameComparator):
             print(f"Error initializing AverageHashComparator: {str(exc)}")
             raise
 
-    def compare(self, static_frame: np.ndarray, new_frame: np.ndarray, stream_key: str) -> Tuple[bool, float]:
+    def compare(self, static_frame: np.ndarray, new_frame: np.ndarray, stream_key: Optional[str] = None) -> Tuple[bool, float]:
         """Compare frames using average hash difference.
         
         Args:
             static_frame: Reference frame (RGB, cv2 image as np.ndarray).
             new_frame: New frame to compare (RGB, cv2 image as np.ndarray).
-            stream_key: Identifier for the video stream (e.g., camera ID).
+            stream_key: Optional identifier for the video stream (e.g., camera ID).
             
         Returns:
             Tuple[bool, float]: (is_similar, hash_difference)
         """
         try:
+            stream_key = stream_key or "default_stream_key"
             if not isinstance(static_frame, np.ndarray) or not isinstance(new_frame, np.ndarray):
                 msg = f"Invalid frame type for stream {stream_key}: static_frame and new_frame must be np.ndarray"
                 logger.error(msg)
-                 #print(msg)
                 return False, 0.0
             if static_frame.shape != new_frame.shape or static_frame.shape[2] != 3:
                 msg = f"Frames for stream {stream_key} have mismatched dimensions or are not RGB"
                 logger.error(msg)
-                 #print(msg)
-                return False, 0.0
-            if not isinstance(stream_key, str) or not stream_key:
-                msg = f"Invalid stream_key for stream {stream_key}: must be a non-empty string"
-                logger.error(msg)
-                 #print(msg)
                 return False, 0.0
             
             static_pil = Image.fromarray(cv2.cvtColor(static_frame, cv2.COLOR_BGR2RGB))
@@ -264,7 +237,6 @@ class PerceptualHashComparator(FrameComparator):
             if not isinstance(threshold, int) or threshold < 0:
                 msg = f"Invalid threshold: {threshold}, must be a non-negative integer"
                 logger.error(msg)
-                 #print(msg)
                 raise ValueError(msg)
             self.threshold = threshold
         except Exception as exc:
@@ -272,32 +244,26 @@ class PerceptualHashComparator(FrameComparator):
             print(f"Error initializing PerceptualHashComparator: {str(exc)}")
             raise
 
-    def compare(self, static_frame: np.ndarray, new_frame: np.ndarray, stream_key: str) -> Tuple[bool, float]:
+    def compare(self, static_frame: np.ndarray, new_frame: np.ndarray, stream_key: Optional[str] = None) -> Tuple[bool, float]:
         """Compare frames using perceptual hash difference.
         
         Args:
             static_frame: Reference frame (RGB, cv2 image as np.ndarray).
             new_frame: New frame to compare (RGB, cv2 image as np.ndarray).
-            stream_key: Identifier for the video stream (e.g., camera ID).
+            stream_key: Optional identifier for the video stream (e.g., camera ID).
             
         Returns:
             Tuple[bool, float]: (is_similar, hash_difference)
         """
         try:
+            stream_key = stream_key or "default_stream_key"
             if not isinstance(static_frame, np.ndarray) or not isinstance(new_frame, np.ndarray):
                 msg = f"Invalid frame type for stream {stream_key}: static_frame and new_frame must be np.ndarray"
                 logger.error(msg)
-                 #print(msg)
                 return False, 0.0
             if static_frame.shape != new_frame.shape or static_frame.shape[2] != 3:
                 msg = f"Frames for stream {stream_key} have mismatched dimensions or are not RGB"
                 logger.error(msg)
-                 #print(msg)
-                return False, 0.0
-            if not isinstance(stream_key, str) or not stream_key:
-                msg = f"Invalid stream_key for stream {stream_key}: must be a non-empty string"
-                logger.error(msg)
-                 #print(msg)
                 return False, 0.0
             
             static_pil = Image.fromarray(cv2.cvtColor(static_frame, cv2.COLOR_BGR2RGB))
@@ -333,7 +299,6 @@ class DifferenceHashComparator(FrameComparator):
             if not isinstance(threshold, int) or threshold < 0:
                 msg = f"Invalid threshold: {threshold}, must be a non-negative integer"
                 logger.error(msg)
-                 #print(msg)
                 raise ValueError(msg)
             self.threshold = threshold
         except Exception as exc:
@@ -341,34 +306,28 @@ class DifferenceHashComparator(FrameComparator):
             print(f"Error initializing DifferenceHashComparator: {str(exc)}")
             raise
 
-    def compare(self, static_frame: np.ndarray, new_frame: np.ndarray, stream_key: str) -> Tuple[bool, float]:
+    def compare(self, static_frame: np.ndarray, new_frame: np.ndarray, stream_key: Optional[str] = None) -> Tuple[bool, float]:
         """Compare frames using difference hash difference.
         
         Args:
             static_frame: Reference frame (RGB, cv2 image as np.ndarray).
             new_frame: New frame to compare (RGB, cv2 image as np.ndarray).
-            stream_key: Identifier for the video stream (e.g., camera ID).
+            stream_key: Optional identifier for the video stream (e.g., camera ID).
             
         Returns:
             Tuple[bool, float]: (is_similar, hash_difference)
         """
         try:
+            stream_key = stream_key or "default_stream_key"
             if not isinstance(static_frame, np.ndarray) or not isinstance(new_frame, np.ndarray):
                 msg = f"Invalid frame type for stream {stream_key}: static_frame and new_frame must be np.ndarray"
                 logger.error(msg)
-                 #print(msg)
                 return False, 0.0
             if static_frame.shape != new_frame.shape or static_frame.shape[2] != 3:
                 msg = f"Frames for stream {stream_key} have mismatched dimensions or are not RGB"
                 logger.error(msg)
-                 #print(msg)
                 return False, 0.0
-            if not isinstance(stream_key, str) or not stream_key:
-                msg = f"Invalid stream_key for stream {stream_key}: must be a non-empty string"
-                logger.error(msg)
-                 #print(msg)
-                return False, 0.0
-            
+
             static_pil = Image.fromarray(cv2.cvtColor(static_frame, cv2.COLOR_BGR2RGB))
             new_pil = Image.fromarray(cv2.cvtColor(new_frame, cv2.COLOR_BGR2RGB))
             diff = dhash(static_pil) - dhash(new_pil)
@@ -385,3 +344,71 @@ class DifferenceHashComparator(FrameComparator):
             logger.error("Unexpected error in DifferenceHash comparison for stream %s: %s", stream_key, str(exc))
             print(f"Unexpected error in DifferenceHash for stream {stream_key}: {str(exc)}")
             return False, 0.0
+
+class HistogramComparator(FrameComparator):
+    """Compare frames using histogram correlation."""
+    
+    def __init__(self, threshold: float = 0.9):
+        """Initialize with threshold for histogram correlation.
+        
+        Args:
+            threshold: Correlation score threshold (default: 0.9).
+        
+        Raises:
+            ValueError: If threshold is not in [0, 1].
+        """
+        try:
+            if not isinstance(threshold, (int, float)) or not 0 <= threshold <= 1:
+                msg = f"Invalid threshold: {threshold}, must be between 0 and 1"
+                logger.error(msg)
+                raise ValueError(msg)
+            self.threshold = threshold
+        except Exception as exc:
+            logger.error("Failed to initialize HistogramComparator: %s", str(exc))
+            print(f"Error initializing HistogramComparator: {str(exc)}")
+            raise
+    
+    def compare(self, static_frame: np.ndarray, new_frame: np.ndarray, stream_key: Optional[str] = None) -> Tuple[bool, float]:
+        """Compare frames using histogram correlation.
+        
+        Args:
+            static_frame: Reference frame (RGB, cv2 image as np.ndarray).
+            new_frame: New frame to compare (RGB, cv2 image as np.ndarray).
+            stream_key: Optional identifier for the video stream (e.g., camera ID).
+            
+        Returns:
+            Tuple[bool, float]: (is_similar, correlation_score)
+        """
+        try:
+            stream_key = stream_key or "default_stream_key"
+            if not isinstance(static_frame, np.ndarray) or not isinstance(new_frame, np.ndarray):
+                msg = f"Invalid frame type for stream {stream_key}: static_frame and new_frame must be np.ndarray"
+                logger.error(msg)
+                return False, 0.0
+            if static_frame.shape != new_frame.shape or static_frame.shape[2] != 3:
+                msg = f"Frames for stream {stream_key} have mismatched dimensions or are not RGB"
+                logger.error(msg)
+                return False, 0.0
+            
+            static_hist = cv2.calcHist([static_frame], [0, 1, 2], None, [8, 8, 8], [0, 256, 0, 256, 0, 256])
+            new_hist = cv2.calcHist([new_frame], [0, 1, 2], None, [8, 8, 8], [0, 256, 0, 256, 0, 256])
+            static_hist = cv2.normalize(static_hist, static_hist).flatten()
+            new_hist = cv2.normalize(new_hist, new_hist).flatten()
+            correlation = cv2.compareHist(static_hist, new_hist, cv2.HISTCMP_CORREL)
+            if np.isnan(correlation):
+                msg = f"NaN detected in Histogram comparison for stream {stream_key}"
+                logger.error(msg)
+                return False, 0.0
+            return correlation > self.threshold, float(correlation)
+        except cv2.error as exc:
+            logger.error("OpenCV error in Histogram comparison for stream %s: %s", stream_key, str(exc))
+            print(f"OpenCV error in Histogram for stream {stream_key}: {str(exc)}")
+            return False, 0.0
+        except (ValueError, TypeError) as exc:
+            logger.error("Value or Type error in Histogram comparison for stream %s: %s", stream_key, str(exc))
+            print(f"Value/Type error in Histogram for stream {stream_key}: {str(exc)}")
+            return False, 0.0
+        except Exception as exc:
+            logger.error("Unexpected error in Histogram comparison for stream %s: %s", stream_key, str(exc))
+            print(f"Unexpected error in Histogram for stream {stream_key}: {str(exc)}")
+            return False, 0.0
