@@ -2,6 +2,7 @@ APP_NAME_TO_USECASE = {
     "people_counting": "people_counting",
     "mask_detection": "mask_detection",
     "vehicle_monitoring": "vehicle_monitoring",
+    "vehicle_type_monitoring": "vehicle_monitoring",
     "weapon_detection": "weapon_detection",
     "traffic_sign_monitoring": "traffic_sign_monitoring",
     "flare_analysis": "flare_analysis",
@@ -28,6 +29,7 @@ APP_NAME_TO_CATEGORY = {
     "people_counting": "general",
     "mask_detection": "mask_detection",
     "vehicle_monitoring": "traffic",
+    "vehicle_type_monitoring": "traffic",
     "weapon_detection": "security",
     "traffic_sign_monitoring": "traffic",
     "flare_analysis": "flare_detection",
