@@ -1,3 +1,5 @@
+import time
+from datetime import datetime, timezone
 """
 Advanced customer service use case implementation.
 
@@ -63,6 +65,94 @@ def assign_person_by_area(detections, customer_areas, staff_areas):
             det['category'] = 'staff'
 
 class AdvancedCustomerServiceUseCase(BaseProcessor):
+    def _format_timestamp_for_video(self, timestamp: float) -> str:
+        """Format timestamp for video chunks (HH:MM:SS.ms format)."""
+        hours = int(timestamp // 3600)
+        minutes = int((timestamp % 3600) // 60)
+        seconds = round(float(timestamp % 60), 2)
+        return f"{hours:02d}:{minutes:02d}:{seconds:04.1f}"
+
+    def _format_timestamp_for_stream(self, timestamp: float) -> str:
+        dt = datetime.fromtimestamp(timestamp, tz=timezone.utc)
+        return dt.strftime('%Y:%m:%d %H:%M:%S')
+
+    def _get_current_timestamp_str(self, stream_info: Optional[Dict[str, Any]], precision=False, frame_id: Optional[str]=None) -> str:
+        """Get formatted current timestamp based on stream type."""
+        if not stream_info:
+            return "00:00:00.00"
+        input_settings = stream_info.get("input_settings", {})
+        if precision:
+            if input_settings.get("start_frame", "na") != "na":
+                if frame_id is not None:
+                    start_time = int(frame_id)/input_settings.get("original_fps", 30)
+                else:
+                    start_time = input_settings.get("start_frame", 30)/input_settings.get("original_fps", 30)
+                return self._format_timestamp_for_video(start_time)
+            else:
+                return datetime.now(timezone.utc).strftime("%Y-%m-%d-%H:%M:%S.%f UTC")
+
+        if input_settings.get("start_frame", "na") != "na":
+            if frame_id is not None:
+                start_time = int(frame_id)/input_settings.get("original_fps", 30)
+            else:
+                start_time = input_settings.get("start_frame", 30)/input_settings.get("original_fps", 30)
+            return self._format_timestamp_for_video(start_time)
+        else:
+            # For streams, use stream_time from stream_info
+            stream_time_str = input_settings.get("stream_info", {}).get("stream_time", "")
+            if stream_time_str:
+                try:
+                    timestamp_str = stream_time_str.replace(" UTC", "")
+                    dt = datetime.strptime(timestamp_str, "%Y-%m-%d-%H:%M:%S.%f")
+                    timestamp = dt.replace(tzinfo=timezone.utc).timestamp()
+                    return self._format_timestamp_for_stream(timestamp)
+                except:
+                    return self._format_timestamp_for_stream(time.time())
+            else:
+                return self._format_timestamp_for_stream(time.time())
+
+    def _get_start_timestamp_str(self, stream_info: Optional[Dict[str, Any]], precision=False) -> str:
+        """Get formatted start timestamp for 'TOTAL SINCE' based on stream type."""
+        if not stream_info:
+            return "00:00:00"
+        input_settings = stream_info.get("input_settings", {})
+        if precision:
+            if input_settings.get("start_frame", "na") != "na":
+                return "00:00:00"
+            else:
+                return datetime.now(timezone.utc).strftime("%Y-%m-%d-%H:%M:%S.%f UTC")
+
+        if input_settings.get("start_frame", "na") != "na":
+            return "00:00:00"
+        else:
+            if not hasattr(self, '_tracking_start_time') or self._tracking_start_time is None:
+                stream_time_str = input_settings.get("stream_info", {}).get("stream_time", "")
+                if stream_time_str:
+                    try:
+                        timestamp_str = stream_time_str.replace(" UTC", "")
+                        dt = datetime.strptime(timestamp_str, "%Y-%m-%d-%H:%M:%S.%f")
+                        self._tracking_start_time = dt.replace(tzinfo=timezone.utc).timestamp()
+                    except:
+                        self._tracking_start_time = time.time()
+                else:
+                    self._tracking_start_time = time.time()
+            dt = datetime.fromtimestamp(self._tracking_start_time, tz=timezone.utc)
+            dt = dt.replace(minute=0, second=0, microsecond=0)
+            return dt.strftime('%Y:%m:%d %H:%M:%S')
+    def get_camera_info_from_stream(self, stream_info):
+        """Extract camera_info from stream_info, matching people_counting pattern."""
+        if not stream_info:
+            return {}
+        # Try to get camera_info directly
+        camera_info = stream_info.get("camera_info")
+        if camera_info:
+            return camera_info
+        # Fallback: try to extract from nested input_settings
+        input_settings = stream_info.get("input_settings", {})
+        for key in ["camera_info", "camera_id", "location", "site_id"]:
+            if key in input_settings:
+                return {key: input_settings[key]}
+        return {}
     def _generate_per_frame_agg_summary(self, processed_data, analytics_results, config, context, stream_info=None):
         """
         Generate agg_summary dict with per-frame incidents, tracking_stats, business_analytics, alerts, human_text.
@@ -110,41 +200,8 @@ class AdvancedCustomerServiceUseCase(BaseProcessor):
             journey_analytics = analytics_results.get("customer_journey_analytics", {})
             business_metrics = analytics_results.get("business_metrics", {})
 
-            # --- Per-frame timestamp logic ---
-            # 1. Try to get from stream_info (video_timestamp)
-            # 2. Else, estimate from frame index and FPS
-            # 3. Else, use real wall-clock elapsed time
-            current_timestamp = None
-            if stream_info:
-                input_settings = stream_info.get("input_settings", {})
-                stream_type = input_settings.get("stream_type", "video_file")
-                if stream_type == "video_file":
-                    # If video_timestamp is a list, use per-frame
-                    video_timestamps = stream_info.get("video_timestamp")
-                    if isinstance(video_timestamps, list) and len(video_timestamps) > idx:
-                        current_timestamp = video_timestamps[idx][:8]
-                    elif isinstance(video_timestamps, str):
-                        current_timestamp = video_timestamps[:8]
-                elif stream_type == "stream":
-                    stream_times = stream_info.get("stream_time")
-                    if isinstance(stream_times, list) and len(stream_times) > idx:
-                        current_timestamp = stream_times[idx][:8]
-                    elif isinstance(stream_times, str):
-                        current_timestamp = stream_times[:8]
-            # If still not found, estimate from frame index and FPS
-            if not current_timestamp or current_timestamp == "00:00:00.00":
-                if fps:
-                    total_seconds = idx / fps
-                elif wallclock_start_time is not None:
-                    total_seconds = time.time() - wallclock_start_time
-                else:
-                    total_seconds = 0
-                hours = int(total_seconds // 3600)
-                minutes = int((total_seconds % 3600) // 60)
-                seconds = int(total_seconds % 60)
-                centiseconds = int((total_seconds - int(total_seconds)) * 100)
-                current_timestamp = f"{hours:02d}:{minutes:02d}:{seconds:02d}.{centiseconds:02d}"
-
+            # --- Per-frame timestamp logic (robust, never default to 00:00:00.00 except first frame) ---
+            current_timestamp = self._get_current_timestamp_str(stream_info, precision=True, frame_id=frame_id)
             start_timestamp = self._get_start_timestamp_str(stream_info)
 
             # --- Alert settings and alerts for each metric ---
@@ -173,7 +230,7 @@ class AdvancedCustomerServiceUseCase(BaseProcessor):
                     }
                 })
             # service efficiency alert
-            efficiency_threshold = getattr(config, "service_efficiency_threshold", 0.5)
+            efficiency_threshold = getattr(config, "service_efficiency_threshold", 0.1)
             if business_metrics.get("service_efficiency", 0) < efficiency_threshold:
                 alert_settings.append({
                     "alert_type": "email",
@@ -247,7 +304,7 @@ class AdvancedCustomerServiceUseCase(BaseProcessor):
                 "alert_settings": alert_settings
             }
             # Harmonize tracking_stats fields with people_counting output
-            camera_info = stream_info.get("camera_info", {}) if stream_info else {}
+            camera_info = self.get_camera_info_from_stream(stream_info)
             input_timestamp = current_timestamp
             reset_timestamp = start_timestamp
             reset_settings = config.create_default_config() if hasattr(config, "create_default_config") else {}
